/*******************************************************************************
* File: ledboard.c
* Purpose: Drive the Raspberry Pi GPIO lines in various patterns to delight
* a viewer of the RPI-LED board. 
*
* Copyright © Microsystem Technologies April, 2020
* http://www.Micronaitc.com/
* Micronatic is a trademark of Microsystem Technologies
* 
* This application uses the generic Linux character devices to control the
* Raspberry Pi gpio pins. The specific driver name and pins used are specific
* to the Raspberry Pi.

* End User Software License Agreement
* The LED light show software developed for the RPI-LED board may be used freely 
* and shared as desired provided the End User Software License Agreement remains
* intact at the top of the software listing. THE SOFTWARE SOURCE CODE IS PROVIDED 
* “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT 
* LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
* FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, ARISING FROM, OUT OF OR IN CONNECTION 
* WITH THE SOFTWARE OR THE USE OF OR OTHER ACTIONS RELATIVE TO THE SOFTWARE CODE.
* 
* Warning: The RPI-LED Light Show Application flashes LEDs ON and OFF. Anyone who 
* is sensitive to flashing lights in any way should not view or operate the RPI-LED
* board Light Show Application. Use at your own risk.
* 
*******************************************************************************/
#include <unistd.h>
#include <dirent.h> 
#include <stdio.h> 
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <linux/gpio.h>
#include <unistd.h>

#define ERROR -1
#define NUM_LED_PINS    24
#define NUM_LED_TOP     12
#define NUM_LED_BOTTOM  12
#define NUM_TIMES       3

/**********************************************************
* Command line build: gcc rpi-led.c -o rpi-led -Wall
**********************************************************/

/* Function prototypes. */
int initOutputs(int **LEDFDList, int *LEDlist, int cnt, int fd);
int initInputs(int *FD1, int *FD2, int fd);
void releaseIO(int *LEDFDList, int cnt, int InFD1, int InFD2);
int gpioSetPin(int fd, int value);
int gpioGetPin(int fd);
int ReadSwitchInputs(int InFD1, int InFD2);

/* LED display function prototypes. */
void LEDBlinkAll(int *iLED_FDs, int OnuSec, int OffuSec, int repeat);
void LEDTopBottomAlternate(int *iLED_FDs, int OnuSec, int OffuSec, int repeat);
void LEDBreatheOut(int *iLED_FDs, int OnuSec, int OffuSec, int repeat);
void LEDBreatheIn(int *iLED_FDs, int OnuSec, int OffuSec, int repeat);
void LEDRoundRobin(int *iLED_FDs, int OnuSec, int OffuSec, int repeat, int dir);
void LEDHalfnHalf(int *iLED_FDs, int OnuSec, int OffuSec, int repeat);
void LEDLeftnRight(int *iLED_FDs, int OnuSec, int OffuSec, int repeat);
void LEDEvenOdd(int *iLED_FDs, int OnuSec, int OffuSec, int repeat);



/*************************************************************************
* Function: main()
* Purpose: Main function entry point for the ledboard application. 
* This function will turn on/off the led board LED's in various patterns.
* 
* Returns: 0 = OKAY, or ERROR
**************************************************************************/
int main(void) 
{
	int retval = 0;
    int Devfd;
    int iLED_FDs[NUM_LED_PINS]; 
    int *pLED_FDs = &iLED_FDs[0];
    int ctrl1FD = 2;
    int ctrl2FD = 5;
    int ctrlSignal = 0;
    
    /* Create list of gpios with LEDs (BMI Numbers). */
    int iLEDs[NUM_LED_PINS] =
    {
        // Left side or bottom row depending on orientation
        3, /* GPIO3) */ 4, 17, 27, 22, 10, 9, 11, 6, 13, 19, 26,  
        // Right side or top row
        14, 15, 18, 23, 24, 25, 8, 7, 12, 16, 20, 21 
    };
	
	printf("\nRunning RPI-LED Board LED Light Show Application\n");
    printf("\nFor RPI-LED project details go to:");
    printf("\nhttp://www.micronatic.com/rpi-led.html\n"); 
    fflush(stdout);  
	
	/* Open the character device driver for access. */
	Devfd = open("/dev/gpiochip0", O_RDONLY);

	if(Devfd != ERROR)
	{  
        printf("\nInitializing...");
        fflush(stdout); 

        /* Gain access to the gpio output pins. */        
        retval = initOutputs(&pLED_FDs, iLEDs, NUM_LED_PINS, Devfd);

        /* Gain access to the two control input pins. */
        retval = initInputs(&ctrl1FD, &ctrl2FD, Devfd);

        /* Set pin on/off example, first LED (GPIO3). */
        gpioSetPin(iLED_FDs[0], 1);
        sleep(1);
        gpioSetPin(iLED_FDs[0], 0);
        sleep(1);

        /* Read switch control input 1 example. */
        ctrlSignal = gpioGetPin(ctrl1FD);
        if(ctrlSignal >= 0)
            printf("\nCTRL Input 1 State: %d", ctrlSignal);
        else
            printf("\nError reading CTRL Input 1.");

        /* Read switch control input 2 example. */
        ctrlSignal = gpioGetPin(ctrl2FD);
        if(ctrlSignal >= 0)
            printf("\nCTRL Input 2 State: %d", ctrlSignal);
        else
            printf("\nError reading CTRL Input 2.");

        printf("\nRunning light show sequence...\n");
        fflush(stdout); 


        /* Program runs forever unles ^C entered at command line or 
        ** control switch input 2 is pulled low by an external switch. */
        while(1)
        {
            /* Blink all LEDs on/off. 200mS on, 100mS off, 5 times. */
            LEDBlinkAll(iLED_FDs, 200000, 100000, 5);

            /* Act on switch control inputs. */
            if(ReadSwitchInputs(ctrl1FD, ctrl2FD)) break;
                      
            /* Blink between top and bottom row. */
            LEDTopBottomAlternate(iLED_FDs, 150000, 75000, 5);

            /* Act on switch control inputs. */
            if(ReadSwitchInputs(ctrl1FD, ctrl2FD)) break;

            /* Breathe in and out. (outward) */
            LEDBreatheOut(iLED_FDs, 85000, 55000, 3);

            /* Act on switch control inputs. */
            if(ReadSwitchInputs(ctrl1FD, ctrl2FD)) break;

            /* Breathe in and out. (inward) */
            LEDBreatheIn(iLED_FDs, 85000, 55000, 3);

            /* Act on switch control inputs. */
            if(ReadSwitchInputs(ctrl1FD, ctrl2FD)) break;

            /* Loop around the board clockwise. */
            LEDRoundRobin(iLED_FDs, 67000, 5500, 3, 0);

            /* Act on switch control inputs. */
            if(ReadSwitchInputs(ctrl1FD, ctrl2FD)) break;

            /* Loop around the board counterclockwise. */
            LEDRoundRobin(iLED_FDs, 67000, 5500, 3, 1);

            /* Act on switch control inputs. */
            if(ReadSwitchInputs(ctrl1FD, ctrl2FD)) break;

            /* Alternat halfs. */
            LEDHalfnHalf(iLED_FDs, 400000, 400000, 5);

            /* Act on switch control inputs. */
            if(ReadSwitchInputs(ctrl1FD, ctrl2FD)) break;

            /* Alternate left and right. */
            LEDLeftnRight(iLED_FDs, 400000, 400000, 5);

            /* Act on switch control inputs. */
            if(ReadSwitchInputs(ctrl1FD, ctrl2FD)) break;

            /* Alternate between even and odd numbered LEDs. */
            LEDEvenOdd(iLED_FDs, 200000, 200000, 6);

            /* Act on switch control inputs. */
            if(ReadSwitchInputs(ctrl1FD, ctrl2FD)) break;
        }
        
        /* Release all file descriptors. */
        releaseIO(iLED_FDs, NUM_LED_PINS, ctrl1FD, ctrl2FD);
        close(Devfd);
    }	
    else
    {	
		/* Failed to open device so print error message. */
		printf("Error %d opening device: /dev/gpiochip0, %s", errno, strerror(errno));		
		
        retval = ERROR;
    }

    printf("\n\nExiting RPI-LED project light show application.\n\n\n");    
    fflush(stdout);
    	
	return retval;
}



/*************************************************************************
* Function: ReadSwitchInputs()
* Purpose:  Read the two switch control inputs.
*
* Receives:
*			int InFD1 - Switch input one FD.
*			int InFD2 - Switch input two FD.
* 
* Returns: 0 = Do nothing else TRUE to exit application.
**************************************************************************/
int ReadSwitchInputs(int InFD1, int InFD2)
{
    int retval = 0;
    
    /* Check for hault sequence switch input. */
    if( (InFD1 != -1) && (gpioGetPin(InFD1) == 0) )
    {
        printf("\nCTRL1 Switch on, halting sequence...");
        fflush(stdout);
        while(gpioGetPin(InFD1) == 0);
    }

    /* Check for exit application switch intput. */
    if( (InFD2 != -1) && (gpioGetPin(InFD2) == 0) )
    {
        printf("\nCTRL2 Switch on, exiting application...");
        fflush(stdout);
        retval = 1;
    }

    return retval;
}


/*************************************************************************
* Function: initOutputs()
* Purpose:  Gain access to the desired gpio pins through the character 
*           device driver.
*
* Receives:
*			int **LEDFDList - Pointer to a list of file descriptors
*           int *LEDlist - List of gpio numbers
*           int cnt - Number of gpio
*           int fd - File descriptor to the character device driver
*
* Returns: OK or ERROR
**************************************************************************/
int initOutputs(int **LEDFDList, int *LEDlist, int cnt, int fd)
{
    int result = 0;
    int gpioline = 0;
    struct gpiohandle_request lineHandle;

    if(cnt < 1) return ERROR;

    /* Gain access to each gpio individually in the event one is taken. */
    for(gpioline = 0; gpioline < cnt; gpioline++)
    {
        /* init to -1 in the event the gpio is not available. */
        (*LEDFDList)[gpioline] = -1;

        /* Clear gpiohandle request structure memory. */
        memset(&lineHandle, 0, sizeof(lineHandle));

        /* Set line values for the next gpio. */
        lineHandle.lineoffsets[0] = LEDlist[gpioline];
		lineHandle.lines = 1;		
        lineHandle.flags = GPIOHANDLE_REQUEST_OUTPUT;	        
        lineHandle.default_values[0] = 0;

        /* Set the name of the consumer of the line. */		
        strncpy(lineHandle.consumer_label, "LED Board (O)", 32);
			   
		/* Get a line handle to the gpio pin. */		
		result = ioctl(fd, GPIO_GET_LINEHANDLE_IOCTL, &lineHandle);
		
        /* If successfull.*/
        if(result != ERROR)        
            (*LEDFDList)[gpioline] = lineHandle.fd; // Save the line fd.
        else if(errno == 16)  // gpio line is in use already.
        {
            printf("\n gpio pin %d is in use.", LEDlist[gpioline]);
            fflush(stdout);  
        }
        else
        {
            printf("\n Error %d opening gpio %d.", errno, LEDlist[gpioline]);
            printf("\n\t %s", strerror(errno));
            fflush(stdout);  
        }
    }

    return 0;
}



/*************************************************************************
* Function: initInputs()
* Purpose:  Gain access to the two control input pins through the character 
*           device driver.
*
* Receives:
*			int *FD1 - BMI GPIO Number on input, GPIO FD on output.
*			int *FD2 - BMI GPIO Number on input, GPIO FD on output.
*           int fd - File descriptor to the character device driver
*
* Returns: OK or ERROR
**************************************************************************/
int initInputs(int *FD1, int *FD2, int fd)
{
    int inputList[2];
    int result = 0;
    int gpioline = 0;
    struct gpiohandle_request lineHandle;

    /* Verify GPIO input (BMI) values. */
    if( (*FD1 < 0 || *FD1 > 27) || (*FD2 < 0 || *FD2 > 27) )
    {
        *FD1 = -1;
        *FD2 = -1;
        return ERROR;
    }

    inputList[0] = *FD1;
    inputList[1] = *FD2;

    /* Init both GPIO input signals. */
    for(gpioline = 0; gpioline < 2; gpioline++)
    {
        /* Clear gpiohandle request structure memory. */
        memset(&lineHandle, 0, sizeof(lineHandle));

        /* Set line values for the next gpio. */
        lineHandle.lineoffsets[0] = inputList[gpioline];
		lineHandle.lines = 1;		
        lineHandle.flags = GPIOHANDLE_REQUEST_INPUT;	        
        lineHandle.default_values[0] = 0;

        /* Set the name of the consumer of the line. */		
        strncpy(lineHandle.consumer_label, "LED Board (I)", 32);
			   
		/* Get a line handle to the gpio pin. */		
		result = ioctl(fd, GPIO_GET_LINEHANDLE_IOCTL, &lineHandle);
		
        /* If successfull.*/
        if(result != ERROR) 
            inputList[gpioline] = lineHandle.fd; // Save the signal fd.
        else if(errno == 16)  // gpio line is in use already.
        {
            printf("\n gpio pin %d is in use.", inputList[gpioline]);
            inputList[gpioline] = -1;
            fflush(stdout);  
        }
        else
        {
            printf("\n Error %d opening gpio %d.", errno, inputList[gpioline]);
            printf("\n\t %s", strerror(errno));
            inputList[gpioline] = -1;
            fflush(stdout);  
        }
    }

    /* Save resulting FDs. */
    *FD1 = inputList[0];
    *FD2 = inputList[1];
    
    return result;
}


/*************************************************************************
* Function: releaseIO()
* Purpose:  Free up the gpio pins previously obtained through the 
*           character device driver.
*
* Receives:
*			int *LEDFDList - List of file descriptors
*           int cnt - Number of FDs in the list.
*           int InFD1 - Control Input one FD.
*           int InFD2 - Control Input two FD.
*
* Returns: void
**************************************************************************/
void releaseIO(int *LEDFDList, int cnt, int InFD1, int InFD2)
{ 
    int gpioline = 0;

    /* Gain access to each gpio individually in the event one is taken. */
    for(gpioline = 0; gpioline < cnt; gpioline++)
    {
        if(LEDFDList[gpioline] != -1)
            close(LEDFDList[gpioline]);
    }

    close(InFD1);
    close(InFD2);
}



/*************************************************************************
* Function: gpioSetPin()
* Purpose: Function to set/clear IO pin value. SET = 3.3V, CLEAR = 0V
*
* Receives:
*			int fd - gpio line handle file descriptor
*           int value = value 0 (zero voltes) or > 0 for 3.3V
*
* Returns: OK or ERROR 
**************************************************************************/
int gpioSetPin(int fd, int value)
{  
	int retval = ERROR;   
	struct gpiohandle_data lineData;

    if(fd != -1)
    {	
        /* Set the gpio line to either 3.3V or 0V. */
        lineData.values[0] = value;    
        retval = ioctl(fd, GPIOHANDLE_SET_LINE_VALUES_IOCTL, &lineData);	    
    }

	return retval;
}



/*************************************************************************
* Function: gpioSetPin()
* Purpose: Function to read IO pin state either 1 (3.3V) or 0 (0V).
*
* Receives:
*			int fd - gpio line handle file descriptor
*           int value = value 0 (zero voltes) or > 0 for 3.3V
*
* Returns: Pin value (0 or 1) else ERROR 
**************************************************************************/
int gpioGetPin(int fd)
{  
	int retval = ERROR;   
	struct gpiohandle_data lineData;

    if(fd != -1)
    {	
        /* Clear gpiohandle data structure memory. */
        memset(&lineData, 0, sizeof(lineData));

        /* Get the current state of the selected line. */
        retval = ioctl(fd, GPIOHANDLE_GET_LINE_VALUES_IOCTL, &lineData);  

        if(retval != ERROR)
        {     
            /* Save the line value 0(LOW) or 1(High). */
            retval = lineData.values[0];
        }        
    }

	return retval;
}




/*************************************************************************
* Function: LEDBlinkAll()
* Purpose:  Blink all LED's on and off at some rate (x) number of times.
*
* Receives:
*           int *iLED_FDs - List of gpio file descriptors.
*           int OnuSec - uSec on time
*           int OffuSec - uSec off time
*           int repeat - number of times to repeat
*
* Returns: void
**************************************************************************/
void LEDBlinkAll(int *iLED_FDs, int OnuSec, int OffuSec, int repeat)
{
    for(int loop = 0; loop < repeat; loop++)
    {
        for(int cnt = 0; cnt < NUM_LED_PINS; cnt++)
            gpioSetPin(iLED_FDs[cnt], 1);
        usleep(OnuSec); 

        for(int cnt = 0; cnt < NUM_LED_PINS; cnt++)
            gpioSetPin(iLED_FDs[cnt], 0);
        usleep(OffuSec); 
    }
}


/*************************************************************************
* Function: LEDTopBottomAlternate()
* Purpose:  Blink all LED's between the top and bottom row (x) number 
*           of times.
*
* Receives:
*           int *iLED_FDs - List of gpio file descriptors.
*           int OnuSec - uSec on time
*           int OffuSec - uSec off time
*           int repeat - number of times to repeat
*
* Returns: void
**************************************************************************/
void LEDTopBottomAlternate(int *iLED_FDs, int OnuSec, int OffuSec, int repeat)
{
    int cnt;

    /* Start with all LEDs off. */
    for(cnt = 0; cnt < NUM_LED_PINS; cnt++)
        gpioSetPin(iLED_FDs[cnt], 0);

    for(int loop = 0; loop < repeat; loop++)
    {
        /* Turn on top row. */
        for(cnt = 0; cnt < NUM_LED_TOP; cnt++)
            gpioSetPin(iLED_FDs[cnt], 1);
        usleep(OnuSec); 

        /* Turn off top row. */
        for(cnt = 0; cnt < NUM_LED_TOP; cnt++)
            gpioSetPin(iLED_FDs[cnt], 0);
        usleep(OffuSec); 

        /* Turn on bottom row. */
        for(cnt = 0; cnt < NUM_LED_BOTTOM; cnt++)
            gpioSetPin(iLED_FDs[cnt+NUM_LED_TOP], 1);
        usleep(OnuSec); 

        /* Turn off bottom row. */
        for(cnt = 0; cnt < NUM_LED_BOTTOM; cnt++)
            gpioSetPin(iLED_FDs[cnt+NUM_LED_TOP], 0);
        usleep(OffuSec); 
    }
}


/*************************************************************************
* Function: LEDBreatheOut()
* Purpose:  Make the top and bottom row of LED's appear to breathe 
*           in and out for (X) number of times; outward direction.
*
* Receives:
*           int *iLED_FDs - List of gpio file descriptors.
*           int OnuSec - uSec on time
*           int OffuSec - uSec off time
*           int repeat - number of times to repeat
*
* Returns: void
**************************************************************************/
void LEDBreatheOut(int *iLED_FDs, int OnuSec, int OffuSec, int repeat)
{
    int cnt;
    int value = 1; 
    int inner, outter;
        

    /* Start with all LEDs off. */
    for(cnt = 0; cnt < NUM_LED_PINS; cnt++)
        gpioSetPin(iLED_FDs[cnt], 0);    

    usleep(OnuSec); 

    for(int loop = 0; loop < repeat; loop++)
    {  
        /* Turn on LEDs in order from center to outside. */
        value = 1;
        for(inner = 5, outter = 6, cnt = 0; cnt < 6; cnt++, outter++, inner--)
        {
            gpioSetPin(iLED_FDs[inner], value); // Top Row
            gpioSetPin(iLED_FDs[outter], value);
            gpioSetPin(iLED_FDs[inner+NUM_LED_TOP], value); // Bottom Row
            gpioSetPin(iLED_FDs[outter+NUM_LED_TOP], value);
            usleep(OnuSec);     
        }

        /* Turn off LEDs in reverse order. */
        value = 0;
        for(inner = 0, outter = 11, cnt = 0; cnt < 6; cnt++, outter--, inner++)
        {
            gpioSetPin(iLED_FDs[inner], value); // Top Row
            gpioSetPin(iLED_FDs[outter], value);
            gpioSetPin(iLED_FDs[inner+NUM_LED_TOP], value); // Bottom Row
            gpioSetPin(iLED_FDs[outter+NUM_LED_TOP], value);
            usleep(OffuSec);
        }                   
    }
}

/*************************************************************************
* Function: LEDBreatheIn()
* Purpose:  Make the top and bottom row of LED's appear to breathe 
*           in and out for (X) number of times; inward direction.
*
* Receives:
*           int *iLED_FDs - List of gpio file descriptors.
*           int OnuSec - uSec on time
*           int OffuSec - uSec off time
*           int repeat - number of times to repeat
*
* Returns: void
**************************************************************************/
void LEDBreatheIn(int *iLED_FDs, int OnuSec, int OffuSec, int repeat)
{
    int cnt;
    int value = 1; 
    int inner, outter;
        

    /* Start with all LEDs off. */
    for(cnt = 0; cnt < NUM_LED_PINS; cnt++)
        gpioSetPin(iLED_FDs[cnt], 0);    

    usleep(OnuSec); 

    for(int loop = 0; loop < repeat; loop++)
    {  
        /* Turn on LEDs from outside inward. */
        value = 1;
        for(inner = 0, outter = 11, cnt = 0; cnt < 6; cnt++, outter--, inner++)
        {
            gpioSetPin(iLED_FDs[inner], value); // Top Row
            gpioSetPin(iLED_FDs[outter], value);
            gpioSetPin(iLED_FDs[inner+NUM_LED_TOP], value); // Bottom Row
            gpioSetPin(iLED_FDs[outter+NUM_LED_TOP], value);
            usleep(OnuSec);  
            
        }                   
    
        /* Turn off LEDs in reverse order. */
        value = 0;
        for(inner = 5, outter = 6, cnt = 0; cnt < 6; cnt++, outter++, inner--)
        {
            gpioSetPin(iLED_FDs[inner], value); // Top Row
            gpioSetPin(iLED_FDs[outter], value);
            gpioSetPin(iLED_FDs[inner+NUM_LED_TOP], value); // Bottom Row
            gpioSetPin(iLED_FDs[outter+NUM_LED_TOP], value);            
            usleep(OffuSec);   
        }
    }
}


/*************************************************************************
* Function: LEDRoundRobin()
* Purpose:  Make two LEDs seem to loop around the board (X) number 
*           of times clockwise.
*
* Receives:
*           int *iLED_FDs - List of gpio file descriptors.
*           int OnuSec - uSec on time
*           int OffuSec - uSec off time
*           int repeat - number of times to repeat
*           int dir - Direction 0 = clockwise, 1 = counterclockwise
*
* Returns: void
**************************************************************************/
void LEDRoundRobin(int *iLED_FDs, int OnuSec, int OffuSec, int repeat, int dir)
{
    int cnt, prevA, prevB;
    int iPosA[] = { 5, 4, 3, 2, 1, 0, 11, 10, 9, 8, 7, 6 };
    int iPosB[] = { 5, 6, 7, 8, 9, 10, 11, 0, 1, 2, 3, 4 };   
    int *pPosA = iPosA;
    int *pPosB = iPosB;

    /* Start with all LEDs off. */
    for(cnt = 0; cnt < NUM_LED_PINS; cnt++)
        gpioSetPin(iLED_FDs[cnt], 0); 

    if(dir)
    {
        pPosA = iPosB;
        pPosB = iPosA;
    }

    prevA = 0;
    prevB = 0;   

    for(int loop = 0; loop < repeat; loop++)
    { 
        for(cnt = 0; cnt < NUM_LED_TOP; cnt++)
        {
            /* Turn off previous two LEDs. */
            gpioSetPin(iLED_FDs[prevA], 0); // Top Row         
            gpioSetPin(iLED_FDs[prevB+NUM_LED_TOP], 0); // Bottom Row
            usleep(OffuSec); 

            /* Turn on next LEDs. */
            gpioSetPin(iLED_FDs[pPosA[cnt]], 1); // Top Row         
            gpioSetPin(iLED_FDs[pPosB[cnt]+NUM_LED_TOP], 1); // Bottom Row
            usleep(OnuSec);

            /* Save current position. */
            prevA = pPosA[cnt];
            prevB = pPosB[cnt];         
        }   
    }
}


/*************************************************************************
* Function: LEDHalfnHalf()
* Purpose:  Lite half of each row with oposing LEDs (X) number of times.
*
* Receives:
*           int *iLED_FDs - List of gpio file descriptors.
*           int OnuSec - uSec on time
*           int OffuSec - uSec off time
*           int repeat - number of times to repeat
*
* Returns: void
**************************************************************************/
void LEDHalfnHalf(int *iLED_FDs, int OnuSec, int OffuSec, int repeat)
{
    int cnt;     

    for(int loop = 0; loop < repeat; loop++)
    {
        for(cnt = 0; cnt < NUM_LED_PINS; cnt++)
        {
            if(cnt < 6 || cnt > 17)
                gpioSetPin(iLED_FDs[cnt], 1);
            else
                gpioSetPin(iLED_FDs[cnt], 0);
        }
        usleep(OnuSec); 

        for(cnt = 0; cnt < NUM_LED_PINS; cnt++)
        {
            if(cnt < 6 || cnt > 17)
                gpioSetPin(iLED_FDs[cnt], 0);
            else
                gpioSetPin(iLED_FDs[cnt], 1);
        }
        usleep(OffuSec); 
    }
}


/*************************************************************************
* Function: LEDLeftnRight()
* Purpose:  Lite LEDs left or right half alternating (X) number of times.
*
* Receives:
*           int *iLED_FDs - List of gpio file descriptors.
*           int OnuSec - uSec on time
*           int OffuSec - uSec off time
*           int repeat - number of times to repeat
*
* Returns: void
**************************************************************************/
void LEDLeftnRight(int *iLED_FDs, int OnuSec, int OffuSec, int repeat)
{
    int cnt; 

    /* Start with all LEDs off. */
    for(cnt = 0; cnt < NUM_LED_PINS; cnt++)
        gpioSetPin(iLED_FDs[cnt], 0); 
    usleep(40000); 

    for(int loop = 0; loop < repeat; loop++)
    {
        for(cnt = 0; cnt < NUM_LED_PINS; cnt++)
        {
            if(cnt < 6 || (cnt > 11 && cnt < 18))
                gpioSetPin(iLED_FDs[cnt], 1);
            else
                gpioSetPin(iLED_FDs[cnt], 0);
        }
        usleep(OnuSec); 

        for(cnt = 0; cnt < NUM_LED_PINS; cnt++)
        {
            if((cnt > 5 && cnt < 12) || cnt > 17)
                gpioSetPin(iLED_FDs[cnt], 1);
            else
                gpioSetPin(iLED_FDs[cnt], 0);
        }
        usleep(OffuSec); 
    }
}


/*************************************************************************
* Function: LEDEvenOdd()
* Purpose:  Alternate even odd numbered LEDs (X) number of times.
*
* Receives:
*           int *iLED_FDs - List of gpio file descriptors.
*           int OnuSec - uSec on time
*           int OffuSec - uSec off time
*           int repeat - number of times to repeat
*
* Returns: void
**************************************************************************/
void LEDEvenOdd(int *iLED_FDs, int OnuSec, int OffuSec, int repeat)
{
    int cnt; 

    /* Start with all LEDs off. */
    for(cnt = 0; cnt < NUM_LED_PINS; cnt++)
        gpioSetPin(iLED_FDs[cnt], 0); 
    usleep(20000); 

    for(int loop = 0; loop < repeat; loop++)
    {
        for(cnt = 0; cnt < NUM_LED_PINS; cnt++)
        {
            if(cnt % 2) 
                gpioSetPin(iLED_FDs[cnt], 1);
            else
                gpioSetPin(iLED_FDs[cnt], 0);
        }
        usleep(OnuSec);         

        for(cnt = 0; cnt < NUM_LED_PINS; cnt++)
        {
            if(cnt % 2) 
                gpioSetPin(iLED_FDs[cnt], 0);
            else
                gpioSetPin(iLED_FDs[cnt], 1);
        }
        usleep(OffuSec); 
    }
}




